;; Proof-of-concept for "shift-mark" commands without using
;; post-command-hook.  The idea is from Johan Bockgård.
;;
;; Currently only works when transient-mark-mode is disabled.
;;
;; If there were was generalized "shift" binding, the individual
;; "*-mark" functions wouldn't be needed.

(defun shift-mark ()
  (unless (region-active-p)
    (push-mark-command nil t))
  (when (or (not transient-mark-mode)
	    (eq 'identity transient-mark-mode))
    (setq transient-mark-mode 'only)))

(defun forward-char-mark (&optional n)
  (interactive "p")
  (shift-mark)
  (forward-char n))

(defun backward-char-mark (&optional n)
  (interactive "p")
  (shift-mark)
  (backward-char n))

(defun forward-word-mark (&optional n)
  (interactive "p")
  (shift-mark)
  (forward-word n))

(defun backward-word-mark (&optional n)
  (interactive "p")
  (shift-mark)
  (backward-word n))

(defun next-line-mark (&optional n)
  (interactive "p")
  (shift-mark)
  (next-line n))

(defun previous-line-mark (&optional n)
  (interactive "p")
  (shift-mark)
  (previous-line n))

(global-set-key [(shift right)] 'forward-char-mark)
(global-set-key [(shift control ?f)] 'forward-char-mark)
(global-set-key [(shift left)] 'backward-char-mark)
(global-set-key [(shift control ?b)] 'backward-char-mark)
(global-set-key [(shift meta f)] 'forward-word-mark)
(global-set-key [(shift control right)] 'forward-word-mark)
(global-set-key [(shift meta b)] 'backward-word-mark)
(global-set-key [(shift control left)] 'backward-word-mark)
(global-set-key [(shift control n)] 'next-line-mark)
(global-set-key [(shift down)] 'next-line-mark)
(global-set-key [(shift control p)] 'previous-line-mark)
(global-set-key [(shift up)] 'previous-line-mark)
